.386p
locals


_TEXT segment dword public use32 'CODE'
    assume cs:_TEXT
    public _far_read32
    public _far_write32
    public _far_memcmp
    public _far_call



_far_read32 proc near

    push ebp
    mov ebp, esp
    push es

    les eax, [ebp + 8]
    mov eax, es:[eax]

    pop es
    pop ebp

    ret

_far_read32 endp



_far_write32 proc near

    push ebp
    mov ebp, esp
    push es
    push ecx

    les eax, [ebp + 8]
    mov ecx, [ebp + 16]
    mov es:[eax], ecx

    pop ecx
    pop es
    pop ebp

    ret

_far_write32 endp



_far_memcmp proc near

    push ebp
    mov ebp, esp
    push es
    push ds
    push esi
    push edi
    push ecx
    push edx

    lds esi, [ebp + 8]
    les edi, [ebp + 16]
    mov ecx, [ebp + 24]

if 0
    ; phase 2: use double-word compares for 4x speedup      (done)
    ;          but need to deal with leftover 0-3 bytes
    ;
    ;?phase 3: do some compares beforehand to cause
    ;          dword alignment of pointers
    ;
    ; phase 4: use discrete instructions for better use
    ;          of 2 integer units in Pentium
    ;
endif

    ; save the remainder in EDX so we don't
    ; have to calculate this later, which
    ; would affect the flags
    ;
    mov edx, ecx
    and edx, 3

    ; shift byte count to convert to dword count
    ;
    shr ecx, 2

    ; force addresses to increment (not decrement)
    ; after each comparison
    ;
    cld

    ; compare until ECX is zero or DWORD's are different
    ;
    repe cmpsd

    ; if we saw a mismatch, skip further compares
    ; and the zero check
    ;
    jnz @2

    ; 0 to 3 extra byte compares
    ;
    mov ecx, edx
    repe cmpsb

@1:
    jnz @2
    xor eax, eax
    jmp @5

@2:
    jb @4
    mov eax, 1
    jmp @5

@4:
    mov eax, -1

@5:
    pop edx
    pop ecx
    pop edi
    pop esi
    pop ds
    pop es
    pop ebp

    ret

_far_memcmp endp



; on entry the stack frame looks like:
;   [EBP +  0]    EBP     of caller we just pushed
;   [EBP +  4]    EIP     to return to
;   [EBP +  8]    EIP     to call
;   [EBP + 12]    CS      to call
;   [EBP + 16]    EBP     to pass
;   [EBP + 20]    EDI     to pass
;   [EBP + 24]    ESI     to pass
;   [EBP + 28]    DS      to pass
;   [EBP + 32]    ES      to pass
;   [EBP + 36]    EDX     to pass
;   [EBP + 40]    ECX     to pass
;   [EBP + 44]    EBX     to pass
;   [EBP + 48]    EAX     to pass
;   [EBP + 52]    EFLAGS  to pass
;
_far_call proc near

    ; set up the stack frame
    ;
    push ebp
    mov ebp, esp

    ; save all the caller's registers
    ;
    pushf
    push eax
    push ebx
    push ecx
    push edx
    push es
    push ds
    push esi
    push edi

    ; push the desired EFLAGS again,
    ; so we can fill them in with IRET
    ;
    push dword ptr [ebp + 52]

    ; push the desired CS:EIP again,
    ; so we can fill them in with IRET
    ;
    push dword ptr [ebp + 12]
    push dword ptr [ebp + 8]

    ; fill in the registers the way the caller wanted us to
    ;
    mov edi, [ebp + 20]
    mov esi, [ebp + 24]
    mov eax, [ebp + 28]
    mov ds, ax
    mov eax, [ebp + 32]
    mov es, ax
    mov edx, [ebp + 36]
    mov ecx, [ebp + 40]
    mov ebx, [ebp + 44]
    mov eax, [ebp + 48]

    ; fill in EBP now that we no longer need to point
    ;
    mov ebp, [ebp + 16]

    ; "return from interrupt" to the routine we were supposed to call
    ;
    iret

    ; restore all the caller's registers
    ;
    pop edi
    pop esi
    pop ds
    pop es
    pop edx
    pop ecx
    pop ebx
    pop eax
    popf

    ; clean up the stack frame
    ;
    pop ebp

    ret

_far_call endp



_TEXT ends

end
