#define MAP_TALKS 0


#include <stdio.h>                  // printf()
#include <dos.h>                    // geninterrupt()
#include "..\types\types.h"
#include "..\dpmi32\phys.h"


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   LINEAR.C - An example of how to map a range of physical memory into     */
/*              your process space so that it can be addressed via a 32-bit  */
/*              near offset from DS.                                         */
/*                                                                           */
/*   Copyright (c) 1994 by Borland International                             */
/*   All Rights Reserved.                                                    */
/*                                                                           */
/*   This source code is hereby released into the public domain.             */
/*   Written by Jeffrey J. Peters                                            */
/*                                                                           */
/*   *** WARNING *** This example is *NOT* guaranteed to work under any      */
/*                   DPMI server except the one that Borland provides in     */
/*                   the DOS PowerPack.  It will definitely *NOT* work       */
/*                   under Windows, NT, Chicago, or OS/2.                    */
/*                                                                           */
/*---------------------------------------------------------------------------*/

// Assembly macros so bcc32 doesn't need to call tasm32 (for speed)
#define LAR_EAX_EBX  {__emit__ (0x0F); __emit__(0x02); __emit__(0xC3);}
#define LSL_EAX_EBX  {__emit__ (0x0F); __emit__(0x03); __emit__(0xC3);}
#define PUSH_EBX     {__emit__ (0x53);}
#define POP_EBX      {__emit__ (0x5B);}




static Bit32 map_helper( Bit32 physical, Bit32 pages, Bit32 *mem_handle_ptr )
{
    static Bit32 offset;
    static Bit32 bytes;
    bytes = pages * PAGE_SIZE;

    _EBX = 0;              // let DPMI pick the address
    _ECX = bytes;          // length in bytes
    _EDX = 0;              // uncommitted
    _AX = 0x0504;          // DPMI 0504h: Allocate Linear Memory Block
    geninterrupt( 0x31 );  // call DPMI
    if( _FLAGS & 1 )
    {
        *mem_handle_ptr = 0;
        return 0;
    }
    offset = _EBX;         // DPMI returns the new linear address
    *mem_handle_ptr = _ESI;    // DPMI returns the handle to this memory block

    // _ESI is already set
    _EBX = 0;              // start at offset 0 into our memory block
    _ECX = pages;          // the number of pages to map
    _EDX = physical;       // the target address range to be mapped in
    _AX = 0x0508;          // DPMI 0508h: Map Device in Memory Block
    geninterrupt( 0x31 );  // call DPMI
    if( _FLAGS & 1 )
    {
        // need to free up the linear block!
        _ESI = *mem_handle_ptr;
        _ESI = _ESI >> 16;
        _EDI = *mem_handle_ptr;        // SI:DI gets 32-bit memory handle
        _AX = 0x0502;                  // DPMI 0502h: Free Linear Memory Block
        geninterrupt( 0x31 );

        *mem_handle_ptr = 0;
        return 0;
    }
    return offset;
}



Bit32 get_limit( Bit16 selector )
{
    _EBX = 0;
    _BX = selector;
    LSL_EAX_EBX;
    return _EAX;
}



static void set_limit( Bit16 selector, Bit32 bytes )
{
    static Bit16 h,l;
    h = (Bit16)(bytes >> 16);
    l = (Bit16)(bytes & 0xFFFF);
    _AX = 8;                      // DPMI 0008h: Set Selector Limit
    _BX = selector;
    _CX = h;
    _DX = l;
    geninterrupt( 0x31 );         // call DPMI
}



Bit32 get_base( Bit16 selector )
{
    Bit16 upper, lower;

    PUSH_EBX;                   // save EBX across this function
    _EBX = 0;
    _EAX = 0;
    _BX = selector;
    _AX = 6;                    // DPMI 0006h: Get Selector Base Address
    geninterrupt( 0x31 );       // call DPMI
    POP_EBX;                    // restore EBX
    if( _FLAGS & 1 )            // carry flag set means error
        return -1;
    upper = _CX;
    lower = _DX;
    return ((Bit32)upper)*0x10000L+lower;
}



// returns the flat pointer you can use to access the physical
// memory address you asked for, regardless of page boundaries
//
void *map_physical( Bit32 physical0, Bit32 bytes, Bit32 *mem_handle_ptr )
{
    Bit32 linear, pages, physical;

    // round the physical address down to a page boundary
    //
    physical = physical0 & 0xFFFFF000;

    // make sure we allocate as many bytes as the user wanted,
    // beyond the physical address he gave us, not just beyond
    // the page-boundary physical address we must start at
    //
    bytes += physical0 - physical;

    // round the byte count up to a page count
    //
    pages = (bytes + PAGE_SIZE - 1) / PAGE_SIZE;

#if MAP_TALKS
    // debug
    printf( "physical to map is 0x%8.8lX\n", physical );
#endif

    // Allocate address space and map in our memory
    linear = map_helper( physical, pages, mem_handle_ptr );
    if( !linear )
        return NULL;

    // may need to adjust the pointer forward in the page
    // so it matches the physical address they asked for,
    // not just the page-boundary physical address that
    // we had to allocate
    //
    linear += physical0 - physical;

#if MAP_TALKS
    // debug
    printf( "new linear address is 0x%8.8lX\n", linear );
    printf( "old DS limit was 0x%8.8lX\n", get_limit(_DS) );
    printf( "old DS base  was 0x%8.8lX\n", get_base(_DS) );
#endif

    // extend the data segment
    set_limit( _DS, get_limit(_DS)+(pages*PAGE_SIZE) );

#if MAP_TALKS
    // debug
    printf( "new DS limit is 0x%8.8lX\n", get_limit(_DS) );
#endif

    return (void *)(linear - get_base(_DS));
}



// WARNING: must unmap memory in reverse of the
// order in which it was mapped, or you get
// EXCEPTION 0xE (stack fault) inside set_limit()
//
void unmap_physical( Bit32 bytes, Bit32 mem_handle )
{
    Bit32 pages = (bytes + PAGE_SIZE - 1) / PAGE_SIZE;

    // this MAY also unmap the device from the memory block
    _ESI = mem_handle;
    _ESI = _ESI >> 16;
    _EDI = mem_handle;             // SI:DI gets 32-bit memory handle
    _AX = 0x0502;                  // DPMI 0502h: Free Linear Memory Block
    geninterrupt( 0x31 );

    // decrease DS's limit again
    set_limit( _DS, get_limit(_DS)-(pages*PAGE_SIZE) );

    // debug
    //printf( "final DS limit is 0x%8.8lX\n", get_limit(_DS) );
}



// returns the linear address of a logical (DS-relative) pointer
//
// how would I convert linear to physical?
// do I have to read the page tables?  what if I am not in ring 0?
//
Bit32 flat_to_linear( void *pointer )
{
    return get_base( _DS ) + (Bit32)pointer;
}



// if an error is returned, none of the memory has been locked
//
// pages that are partially overlapped at beginning and end
// will be locked
//
// a lock count is maintained per page, so you must unlock as
// many times as you lock
//
// returns successfully and does nothing on DPMI hosts that
// do not support virtual memory
//
// DPMI hosts that support virtual memory may also ignore
// this function, but they must be able to deal with page
// at any time, including during an interrupt or exception
// handler
//
// possible errors:
//      0x8013      physical memory unavailable
//      0x8017      lock count exceeded
//      0x8025      invalid linear address (unallocated pages)
//
Bit16 lock_linear( Bit32 pointer, Bit32 length )
{
    _BX = pointer >> 16;
    _CX = pointer & 65535;
    _SI = length >> 16;
    _DI = length & 65535;
    _AX = 0x0600;
    geninterrupt( 0x31 );

    if( _FLAGS & 1 )
        return _AX;

    return 0;
}



// if an error is returned, none of the memory has been unlocked
//
// pages that are partially overlapped at beginning and end
// will be unlocked
//
// a lock count is maintained per page, so you must unlock as
// many times as you locked
//
// returns successfully and does nothing on DPMI hosts that
// do not support virtual memory
//
// DPMI hosts that support virtual memory may also ignore
// this function, but they must be able to deal with page
// at any time, including inside interrupt or exception
// handlers
//
// possible errors:
//      0x8002      invalid state (page not locked)
//      0x8025      invalid linear address (unallocated pages)
//
Bit16 unlock_linear( Bit32 pointer, Bit32 length )
{
    _BX = pointer >> 16;
    _CX = pointer & 65535;
    _SI = length >> 16;
    _DI = length & 65535;
    _AX = 0x0601;
    geninterrupt( 0x31 );

    if( _FLAGS & 1 )
        return _AX;

    return 0;
}



