#include <stdlib.h>                 /* exit() */
#include <stdio.h>                  /* printf() FILE */
#include <string.h>                 /* strlen() strcat() */
#include "..\integrat\subnet.h"     /* Subnet SubnetTree boolean */
#include "..\wattcp\tcp.h"          /* inet_ntoa() */
#include "..\intel\byteswap.h"      /* htonl() */
#include "..\integrat\constant.h"   /* DO_FOREGROUND_STATS */
#include "..\integrat\flat.h"       /* flat_alloc() */



// none of this code gets compiled in
// if it's isn't going to be called
//
#if DO_FOREGROUND_STATS



/* head of linked list
 */
static SubnetTree *subnet_head = NULL;

/* how many nodes should be in the tree now?
 */
static unsigned subnet_node_count = 0;

/* lookup tables
 */
static unsigned msb_of_byte[ 256 ];
static unsigned lsb_of_byte[ 256 ];



/* finds out the least significant set bit
 *
 * counts most significant bit as length*8 - 1, least as 0
 *
 * returns length*8 if no bits are set
 *
 * assumes that argument is in network byte order
 * (most significant first)
 */
unsigned lsb_of_array( Bit8 *array, unsigned length )
{
    register unsigned byte_count;
    register Bit8 byte;

    for( byte_count=length; byte_count>0; byte_count-- )
    {
        byte = array[ byte_count  - 1 ];
        if( byte != 0 )
            return (8 * (length - byte_count)) + lsb_of_byte[ byte ];
    }

    return( 8 * length );
}



/* finds out the least significant set bit
 *
 * counts most significant bit as 31, least as 0
 *
 * returns MAX_ADDR_BITS if no bits are set
 *
 * assumes that argument is host's interpretation
 * of a value that was in network byte order
 */
static unsigned lsb_of_long( Bit32 value )
{
    register unsigned byte_count;
    union {
        Bit8  b_value[ 4 ];
        Bit32 l_value;
    } merged;
    register Bit8 byte;

    merged.l_value = value;
    for( byte_count=4; byte_count>0; byte_count-- )
    {
        byte = merged.b_value[ byte_count  - 1 ];
        if( byte != 0 )
            return (8 * (4 - byte_count)) + lsb_of_byte[ byte ];
    }

    return MAX_ADDR_BITS;
}



/* finds out the most significant set bit
 *
 * counts most significant bit as 31, least as 0
 *
 * returns MAX_ADDR_BITS if no bits are set
 *
 * assumes that argument is in network byte order
 */
static unsigned msb_of_long( Bit32 value )
{
    register unsigned byte_count;
    union {
        Bit8  b_value[ 4 ];
        Bit32 l_value;
    } merged;
    register Bit8 byte;

    merged.l_value = value;
    for( byte_count=0; byte_count<4; byte_count++ )
    {
        byte = merged.b_value[ byte_count ];
        if( byte != 0 )
            return (8 * (3 - byte_count)) + msb_of_byte[ byte ];
    }

    return MAX_ADDR_BITS;
}



/* does any initialization necessary for this module
 */
void InitSubnet( void )
{
    unsigned count, bit;
    unsigned min, max;
    unsigned step;
    Bit32 addr, addr2;                  /* debug for joel */
    int result;                         /* debug for joel */

    msb_of_byte[ 0 ] = 8;               /* indicates an error */
    min = 1;
    max = 2;
    for( bit=0; bit<8; bit++ )
    {
        for( count=min; count<max; count++ )
            msb_of_byte[ count ] = bit;

        min = max;
        max <<= 1;
    }

    lsb_of_byte[ 0 ] = 8;               /* indicates an error */
    min = 1;
    step = 2;
    for( bit=0; bit<8; bit++ )
    {
        for( count=min; count<256; count+=step )
            lsb_of_byte[ count ] = bit;

        min = step;
        step <<= 1;
    }

#if 0
    /* debug for joel */
    for( count=0; count<256; count++ )
        printf( "msb_of_byte[%d] = %d\n", count, msb_of_byte[count] );

    for( count=0; count<256; count++ )
        printf( "lsb_of_byte[%d] = %d\n", count, lsb_of_byte[count] );

    /* debug for joel */
    /* step a set bit from lsb to msb and see if the
     * routines identify it properly
     */
    for( addr=1,count=0; count<MAX_ADDR_BITS; count++,addr<<=1 )
    {
        addr2 = htonl( addr );
        result = lsb_of_long( addr2 );
        if( result != count )
            printf( "subnet internal error: lsb_of_long(%d) should be %d, is %d\n",
                addr, count, result );

        result = msb_of_long( addr2 );
        if( result != count )
            printf( "subnet internal error: msb_of_long(%d) should be %d, is %d\n",
                addr, count, result );
    }

    /* debug for joel */
    /* make sure zero (no bits set at all) causes
     * -1 to be returned from both functions
     */
    result = lsb_of_long( 0 );
    if( result != MAX_ADDR_BITS )
        printf( "subnet internal error: lsb_of_long(0) should be %d, is %d\n",
            MAX_ADDR_BITS, result );

    result = lsb_of_long( 0 );
    if( result != MAX_ADDR_BITS )
        printf( "subnet internal error: msb_of_long(0) should be %d, is %d\n",
            MAX_ADDR_BITS, result );
#endif
}



/* assumes that spot is not NULL
 *
 * this is unaffected by how we number bits
 *
 * use this family of functions any time you traverse
 * the tree so that the decision about what "zero" and
 * "one" fields represent is encapsulated, and can change easily
 */

/* old (presumably slow) code was:
 *
 * static SubnetTree *NextSubnetTree( SubnetTree *spot, Bit32 addr )
 * {
 *    if( addr & spot->mask )
 *        return spot->one;
 *    else
 *        return spot->zero;
 * }
 */
/* even though this is a C macro, you still cannot use
 * it as an LVALUE (the left side of an assignment)
 *
 * a function returning a C++ reference would have worked
 */
#define NextSubnetTree(spot,addr) ((addr)&(spot)->mask) ? (spot)->one : (spot)->zero

/* cannot have an else statement after an invocation
 * of this macro in your code, unless you put curly
 * braces around your invocation or omit the semicolon
 * after the call, because that semicolon looks like an
 * empty statement to the compiler
 */
#define NextSubnetTreeAssign1(spot,addr,this) \
{ \
    if( (addr)&((spot)->mask) ) \
        (spot)->one  = this; \
    else \
        (spot)->zero = this; \
}

/* same problem with else statements as NextSubnetTreeAssign1()
 */
#define NextSubnetTreeAssign2(spot,addr,this,other) \
{ \
    if( (addr)&((spot)->mask) ) \
    { \
        (spot)->one  = this; \
        (spot)->zero = other; \
    } \
    else \
    { \
        (spot)->one  = other; \
        (spot)->zero = this; \
    } \
}



/* assumes that IP address passed is in network byte order
 */
static Subnet *FindSubnetRecurse( Bit32 address, SubnetTree *node )
{
    Bit32 mask;
    Subnet *is_child;

    if( !node )
        return NULL;

    is_child = FindSubnetRecurse( address, NextSubnetTree( node, address ) );

    if( is_child )
        return is_child;

    if( !node->external )
        return NULL;

    if( (address & node->external->mask) == node->external->addr )
        return node->external;

    return NULL;
}



/* gets address and mask in network byte order
 */
Subnet *FindSubnet( register Bit32 address )
{
    return FindSubnetRecurse( address, subnet_head );
}



/* assumes Subnet address and mask fields are in network byte order
 *
 * allows zero masks
 *
 * assumes memory pointed to by "external" argument
 * was allocated from heap and can be owned by us afterwards
 *
 * returns TRUE if successful, FALSE otherwise
 */
boolean InsertSubnet( Subnet *external )
{
    int bit, diff_bit;                  /* must be signed to allow host routes! */
    Bit32 addr;
    SubnetTree *spot, *bottom_spot, *next_spot;
    SubnetTree *new_spot, *new_spot2;
    char temp[ 100 ];                   /* so we can call inet_ntoa() */

    /* make sure the caller does not have any bits set
     * in their address that are cleared in their mask
     * (enforce the mask)
     */
    external->addr &= external->mask;

    /* point at the bit beyond (less significant than)
     * the least significant set bit of mask
     *
     * if no bits on in mask (default route), bit will
     * be 31 (most significant)
     *
     * if all bits on in mask (host route), bit will
     * be -1 (less than least significant)
     */
    bit = lsb_of_long( external->mask ) - 1;

    addr = external->addr;

    /* if there is nothing in the tree...
     */
    if( !subnet_head )
    {
        new_spot = (SubnetTree *) flat_alloc( sizeof( SubnetTree ) );
        if( !new_spot )
            return FALSE;
        subnet_node_count++;

        new_spot->zero = NULL;
        new_spot->one = NULL;
        new_spot->parent = NULL;       /* this is root node */
        new_spot->bit = bit;
        new_spot->mask = htonl( 1 << bit );
        new_spot->external = external;
        subnet_head = new_spot;

        /* already done... amazing!
         */
        return TRUE;
    }

    /* first look for a tree node which would test a less
     * significant bit, and which has external data attached
     *
     * note that less-significant bits are further from the
     * root node
     */
    for( spot=subnet_head; ; spot=next_spot )
    {
        next_spot = NextSubnetTree( spot, addr );

        /* if this node tests the same or less significant
         * bit, and it has data attached, stop here
         */
        if( (spot->bit <= bit) && spot->external )
            break;

        /* if we would go nowhere from this split,
         * stop here
         */
        if( !next_spot )
        {
            /* consistency check */
            if( !spot->external )
            {
                printf( "subnet internal error: no child on our path,"
                    " but no external data is attached!\n" );

                return FALSE;
            }

            /* we still do not know we will be inserting below
             * this guy, because if we differ from him at a more
             * significant bit than his mask length, we will want
             * to insert above him
             */

            break;
        }
    }

    /* find out the most significant differing bit
     */
    diff_bit = msb_of_long( addr ^ spot->external->addr );

    /* if this node tests the same bit,
     * and IP address is the same,
     * it is a duplicate
     */
    if( diff_bit==MAX_ADDR_BITS && bit==spot->bit )
    {
        printf( "subnet %s 0x%X '%s' originally inserted by line %d found; new insertion"
            " from line %d ignored\n", inet_ntoa(temp,external->addr),
            external->mask, external->as_path,
            spot->external->line, external->line );
        return TRUE;
    }

    /* if no bits differ which are common to both masks...
     */
    if( diff_bit==MAX_ADDR_BITS || diff_bit<=bit || diff_bit<=spot->bit )
    {
        /* the differences in mask length will determine
         * which routing takes precedence, so act as if
         * the first bit beyond our mask was different
         */
        diff_bit = bit;
    }

    /* remember the bottom spot, in case we need its
     * data for comparison purposes later
     */
    bottom_spot = spot;

    /* step back up the tree until we get to an old node
     * which is checking the same or more significant bit
     */
    while( diff_bit > spot->bit )
    {
        /* cannot go up above root node
         */
        if( spot->parent == NULL )
            break;

        spot = spot->parent;
    }

    /* if we are at root node, it must
     * test too insignificant a bit...
     */
    if( diff_bit > spot->bit )
    {
        /* consistency check */
        if( spot != subnet_head )
        {
            printf( "subnet internal error: node with no parent is not root!\n" );
            exit( 1 );
        }

        /* we must insert above current node,
         * becoming the new root node
         *
         * but we still don't know whether we will attach
         * external data here or on a new child node
         */
        new_spot = (SubnetTree *) flat_alloc( sizeof( SubnetTree ) );
        if( !new_spot )
            return FALSE;
        subnet_node_count++;

        new_spot->zero = NULL;
        new_spot->one = NULL;
        new_spot->parent = NULL;       /* this is root node */
        new_spot->bit = diff_bit;
        new_spot->mask = htonl( 1 << diff_bit );
        subnet_head = new_spot;

        /* reuse the code below that deals with 2nd nodes
         */
        next_spot = spot;
    }

    /* if the old node is checking a more significant bit...
     */
    else if( diff_bit < spot->bit )
    {
        /* if there is a node below, it must be checking too
         * insignificant a bit, or we would have stopped there
         *
         * so we want to at least put a node below this one
         *
         * but we still don't know whether we will attach
         * external data here or on a new child node
         */
        new_spot = (SubnetTree *) flat_alloc( sizeof( SubnetTree ) );
        if( !new_spot )
            return FALSE;
        subnet_node_count++;

        new_spot->zero = NULL;
        new_spot->one = NULL;
        new_spot->parent = spot;
        new_spot->bit = diff_bit;
        new_spot->mask = htonl( 1 << diff_bit );

        /* it is safe to use our address here because
         * we are guaranteed that it is checking a bit
         * which is inside our mask
         */
        next_spot = NextSubnetTree( spot, addr );
        NextSubnetTreeAssign1( spot, addr, new_spot );
    }

    /* old node must have been checking the bit we want
     */
    else
    {
        /* old node should take the place of
         * the split we would have created
         */
        new_spot = spot;

        /* it is safe to use our address here, because
         * we do not use next_spot below if diff_bit==bit
         */
        next_spot = NextSubnetTree( spot, addr );
    }

    /* now do we also have to insert a 2nd (child) node?
     *
     * not if the difference bit is outside one of the masks
     */
    if( diff_bit == bit )
    {
        if( new_spot == spot )
        {
            /* consistency check */
            if( spot->external )
            {
                printf( "subnet internal error: old spot had exact bit we"
                    " wanted (%d), and it was outside the new mask, but the"
                    " external data was already filled in!\n", diff_bit );

                return FALSE;
            }
        }
        else
        {
            /* we do not have the bit in our address
             * to compare; use the address we saw below
             */
            NextSubnetTreeAssign1( new_spot, bottom_spot->external->addr, next_spot );
            if( next_spot )
                next_spot->parent = new_spot;
        }

        new_spot->external = external;
    }
    else
    {
        /* consistency check */
        if( new_spot == spot )
            if( next_spot )
            {
                printf( "subnet internal error: old spot had exact bit we"
                    " wanted (%d), and it was inside (%d) the new mask, but the"
                    " next pointer was already filled in!\n", diff_bit, bit );

                return FALSE;
            }

        new_spot2 = (SubnetTree *) flat_alloc( sizeof( SubnetTree ) );
        if( !new_spot2 )
            return FALSE;
        subnet_node_count++;

        new_spot2->zero = NULL;
        new_spot2->one = NULL;
        new_spot2->parent = new_spot;
        new_spot2->bit = bit;
        new_spot2->mask = htonl( 1 << bit );
        new_spot2->external = external;

        if( new_spot == spot )
        {
            /* we are overwriting next_spot in an existing split
             * to point at our new node
             */

            /* safe to use our own address because we already
             * know that new_spot->bit is inside our mask
             * because diff_bit != bit
             */
            NextSubnetTreeAssign1( new_spot, addr, new_spot2 );
        }
        else
        {
            /* we are pointing a new split at a new node,
             * and pointing the other path at the old nodes
             */

            new_spot->external = NULL;

            /* safe to use our own address because we already
             * know that new_spot->bit is inside our mask
             * because diff_bit != bit
             */
            NextSubnetTreeAssign2( new_spot, addr, new_spot2, next_spot );
            if( next_spot )
                next_spot->parent = new_spot;
        }
    }

    /* done... finally!
     */
    return TRUE;
}



#define SUBNET_LINE_LENGTH sizeof(" 123.123.123.123\t\t0x01020304\t\t'12345 12345 12345 12345 12345 i'\t\t 23456\n")

/* prints out each node, in order of preference for routing
 */
static void WalkSubnetsRecurse( char *buffer, unsigned max_count, unsigned *count, SubnetTree *node )
{
    int width;
    char temp[ 16 ];

    if( !node )
        return;

    if( *count + SUBNET_LINE_LENGTH > max_count )
        return;

    /* this (depth first, zero first) printout ordering of children
     * is arbitrary; actual routing lookup does not care
     */
    WalkSubnetsRecurse( buffer, max_count, count, NextSubnetTree( node, 0 ) );
    WalkSubnetsRecurse( buffer, max_count, count, NextSubnetTree( node, -1 ) );

    if( !node->external )
        return;

    inet_ntoa( temp, ntohl( node->external->addr ) );

    sprintf( buffer+*count, "%-16s 0x%.8X %6d '%s'\n%n",
        temp,
        ntohl( node->external->mask ),
        node->external->line,
        node->external->as_path,
        &width );
    (*count) += width;
}



/* place one line into buffer for each routing
 *
 * they will appear in order of preference
 */
unsigned WalkSubnets( char *buffer, unsigned max_size )
{
    SubnetTree *node;
    unsigned count;

    if( !subnet_head )
    {
        strcpy( buffer, "No subnets\n" );
        return strlen( buffer );
    }

    strcpy( buffer, "Subnetted nets, masks, AS paths, and line numbers:\n" );
    count = strlen( buffer );
    WalkSubnetsRecurse( buffer, max_size, &count, subnet_head );

    return count;
}


/* read-only access to global variable
 */
unsigned GetSubnetNodeCount( void )
{
    return subnet_node_count;
}



#endif  // DO_FOREGROUND_STATS
