#ifndef _SUBNET_H
#define _SUBNET_H



/* headers we depend upon
 */
#include "..\types\types.h"



/* manifest constants
 */
#define MAX_ADDR_BITS       32
#define MAX_AS_PATH_WIDTH   20


typedef int boolean;

typedef struct Subnet_ Subnet;
struct Subnet_
{
    Bit32 addr;             /* in network byte order */
    Bit32 mask;             /* in network byte order */
    Bit32 nexthop_addr;     /* our router that peers with exit AS */
    char *as_path;          /* allocated from heap so free me later! */
    unsigned line;          /* where in routing table was this?  (aids user debugging) */
    unsigned short src_as;  /* source AS number (owner of subnet) */
    unsigned short exit_as; /* exit AS number (whose network gets it after us) (may be same as source AS) */
};

/* radix trie node used internally for Subnet lookup
 */
typedef struct SubnetTree_ SubnetTree;
struct SubnetTree_
{
   SubnetTree *zero;        /* if bit is zero */
   SubnetTree *one;         /* if bit is one  */
   SubnetTree *parent;      /* for backtracing, NULL if root of tree */
   int bit;                 /* which bit to test, counting 0 as lsb       must be signed to allow host routes! */
   Bit32 mask;              /* speeds up the single-bit test; in network byte order */
   Subnet *external;        /* if non NULL, the address and mask */
};


/* prototypes for exported support functions
 */
#if 1
    unsigned lsb_of_array( Bit8 *array, unsigned length );
    void InitSubnet( void );
    Subnet *FindSubnet( register Bit32 address );
    boolean InsertSubnet( Subnet *external );
    unsigned WalkSubnets( char *buffer, unsigned max_size );
    unsigned GetSubnetNodeCount( void );
#else
    unsigned lsb_of_array();
    void InitSubnet();
    Subnet *FindSubnet();
    boolean InsertSubnet();
    unsigned WalkSubnets();
    unsigned GetSubnetNodeCount();
#endif



#endif      /* not included yet */
