#ifndef _TRACE_H
#define _TRACE_H

#include "..\intel\timestmp.h"          // class PentiumClock
#include "..\fore\cardstmp.h"           // class CardClock
#include "..\aal5\atm.h"                // struct AtmHeader
#include "..\wattcp\tcp.h"              // typedef u_long u_short u_char
#include "..\integrat\constant.h"
#include "..\integrat\ip.h"
#include "..\integrat\ip_icmp.h"
#include "..\integrat\igmp.h"
#include "..\integrat\tcp.h"
#include "..\integrat\udp.h"



/*
 * tracerecord actually appears in the raw trace data
 */
struct tracerec
{
    CardClock when;

    AtmHeaderCoerce atm;

#if CELLS_PER_BUFFER > 0
    struct {
        union {
            struct {
                u_char dsap;      /* (destination services access point) */
                u_char ssap;      /* (source services access point) */
            } lsap_pieces;

            u_short lsap;         /* (link services access point) */
#define LSAP_SNAP     0xAAAA
        };
        u_char ctl;               /* (control) */
#define CTL_SNAP      0x03
    } tr_llc;                     /* 0xAA 0xAA 0x03 (SNAP header follows) */

    struct {
        u_char oui[ 3 ];          /* 0x00 0x00 0x00 (org. unique ID) */
        u_short ethertype;        /* 0x08 0x00 (IP) */
#define ETHERTYPE_IP  0x0800
#define ETHERTYPE_ARP 0x0803      /* anything that isn't IP is fine here */
    } tr_snap;

    struct ip tr_ip;

    /* IP options would go here */

    union
    {
        struct icmp   tr_icmp;
        struct igmp   tr_igmp;
        struct tcphdr tr_tcp;
        struct udphdr tr_udp;
    };

    /* TCP options would go here */

#define tr_etype    tr_snap.ethertype
#define tr_src      tr_ip.ip_src.s_addr
#define tr_dst      tr_ip.ip_dst.s_addr
#define tr_offset   tr_ip.ip_off
#define tr_proto    tr_ip.ip_p
#define tr_tcpflags tr_tcp.th_flags
#define tr_dport    tr_tcp.th_dport
#define tr_sport    tr_tcp.th_sport
#define tr_len      tr_ip.ip_len
#define tr_tos      tr_ip.ip_tos

#endif
};




struct traceblock
{
    // how many of buffers in this block are filled in?
    //
    Bit32 used;

    // which card filled in this block?
    //

    // PCI bus handle of the card
    //
    Bit16 cardnum;

    // global array index of the card
    //
    Bit8 cardindex;

    // must fill up a disk sector, otherwise write is 40 times slower!
    //
    Bit8 padding[ SECTOR_SIZE - sizeof(Bit8) - sizeof(Bit16) - sizeof(Bit32) ];

    // maybe not all these are used
    //
    struct tracerec data[ COMPL_RING_ENTRIES ];
};



#endif      /* not included yet */
