#  1715, Sat 3 May 97
#
#  Makefile for OC3MON Waterloo TCP/IP kernel (flat memory model)
#

DEBUG =     D            # E to enable, D to disable
PROCESSOR = 5            # 5 for Pentium only
CC = bcc32 -WX           # Flat memory model
ASM = tasm32 -dFUNC_L=0  # 32-bit Assembler

!if '$(DEBUG)'=='E'
CDEBUG= -v
ADEBUG= -zi
TEXTDEBUG=enabled
!elif '$(DEBUG)'=='D'
TEXTDEBUG=disabled
!else
!error  DEBUG must be set to either E or D
!endif

!if '$(PROCESSOR)'=='5'
PROC= -5
APROC= -j.586P
!else
!error  PROCESSOR must be set to 5
!endif

CFLAGS= $(PROC) -w-par -a-  -Otcgabeimpv -O -Z  $(CDEBUG)

AFLAGS= $(APROC) -mx -t -i..\wattcp $(ADEBUG)

LIBNAME = wattcp

.c.obj:
    $(CC) $(CFLAGS) -c $*.c
    tlib $(LIBNAME) -+$@

.cpp.obj:
    $(CC) $(CFLAGS) -c $*.cpp
    tlib $(LIBNAME) -+$@

.asm.obj:
    $(ASM) $(AFLAGS) $*
    tlib $(LIBNAME) -+$@

SRC = ..\wattcp

wt_objs= pcicmp.obj \
        bsdname.obj fragment.obj inchksum.obj semaphor.obj netback.obj \
        outhexes.obj pc_cbrk.obj pcarp.obj pcbootp.obj pcbsd.obj \
        pcbuf.obj pcconfig.obj pcdbug.obj pcpkt.obj \
        pcsed.obj pcstat.obj pctcp.obj sock_dbu.obj \
        sock_ini.obj sock_prn.obj udp_dom.obj udp_nds.obj \
        timeout.obj npktrace.obj test.obj

all:  upcalls.obj $(wt_objs)

wattcp.lib: $(objs) upcalls.obj
        echo done

upcalls.obj:    $(SRC)\upcalls.c $(SRC)\pktdrv.h ..\dpmi32\realmode.h
    $(CC) $(CFLAGS) -c $(SRC)\upcalls.c  # Don't put this into the library

test.obj:       $(SRC)\test.c $(SRC)\wattcp.h
bsdname.obj:    $(SRC)\bsdname.c $(SRC)\wattcp.h
fragment.obj:   $(SRC)\fragment.c $(SRC)\wattcp.h
inchksum.obj:   $(SRC)\inchksum.asm \
                    $(SRC)\macros.hsm $(SRC)\masmdefs.hsm $(SRC)\model.hsm
semaphor.obj:   $(SRC)\semaphor.asm \
                    $(SRC)\macros.hsm $(SRC)\masmdefs.hsm $(SRC)\model.hsm
netback.obj:    $(SRC)\netback.c $(SRC)\tcp.h
outhexes.obj:   $(SRC)\outhexes.c
pc_cbrk.obj:    $(SRC)\pc_cbrk.c $(SRC)\wattcp.h
pcarp.obj:      $(SRC)\pcarp.c $(SRC)\wattcp.h
pcbootp.obj:    $(SRC)\pcbootp.c $(SRC)\wattcp.h $(SRC)\bootp.h
pcbsd.obj:      $(SRC)\pcbsd.c $(SRC)\wattcp.h
pcbuf.obj:      $(SRC)\pcbuf.c $(SRC)\wattcp.h
pcconfig.obj:   $(SRC)\pcconfig.c $(SRC)\wattcp.h
pcdbug.obj:     $(SRC)\pcdbug.c $(SRC)\wattcp.h
pcpkt.obj:      $(SRC)\pcpkt.c $(SRC)\wattcp.h $(SRC)\ethdev.h \
                    $(SRC)\elib.h $(SRC)\pktdrv.h ..\dpmi32\realmode.h
pcsed.obj:      $(SRC)\pcsed.c $(SRC)\wattcp.h $(SRC)\ethdev.h
pcstat.obj:     $(SRC)\pcstat.c $(SRC)\wattcp.h
pctcp.obj:      $(SRC)\pctcp.c $(SRC)\wattcp.h
sock_dbu.obj:   $(SRC)\sock_dbu.c $(SRC)\wattcp.h
sock_ini.obj:   $(SRC)\sock_ini.c $(SRC)\wattcp.h
sock_prn.obj:       $(SRC)\sock_prn.c $(SRC)\wattcp.h
udp_dom.obj:    $(SRC)\udp_dom.c $(SRC)\tcp.h $(SRC)\elib.h
udp_nds.obj:    $(SRC)\udp_nds.c $(SRC)\wattcp.h

timeout.obj:    $(SRC)\timeout.cpp ..\intel\timestmp.h
npktrace.obj:   $(SRC)\npktrace.cpp $(SRC)\wattcp.h ..\intel\timestmp.h
pcicmp.obj:     $(SRC)\pcicmp.cpp $(SRC)\wattcp.h ..\intel\timestmp.h

clean:
    del err*.*
    del $(LIBNAME).lib
    del $(LIBNAME).bak
    del pcicmp.obj
    del bsdname.obj
    del fragment.obj
    del inchksum.obj
    del semaphor.obj
    del netback.obj
    del outhexes.obj
    del pc_cbrk.obj
    del pcarp.obj
    del pcbootp.obj
    del pcbsd.obj
    del pcbuf.obj
    del pcconfig.obj
    del pcdbug.obj
    del pcpkt.obj
    del pcpkt16.obj
    del asmpkt_h.obj
    del pcsed.obj
    del pcstat.obj
    del pctcp.obj
    del sock_dbu.obj
    del sock_ini.obj
    del sock_prn.obj
    del udp_dom.obj
    del udp_nds.obj
    del timeout.obj
    del npktrace.obj
    del test.obj
    del upcalls.obj
